/**********************************************************

This class waits for a certain process to exit.

Supported by any OS that has "tasklist" process which
outputs list of running tasks.

This custom code was contributed to Acresso's Custom
Code Download website by Carlos O'Donell at Codesourcery.

***********************************************************/

import com.zerog.ia.api.pub.*;
import java.lang.Runtime;
import java.lang.Thread;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

public class WaitForProcessExit extends CustomCodeAction
{
	// Sleep during to avoid busy-wait.
	static final int sleep_1s = 1000;

	public String getInstallStatusMessage()
	{
		// Display this message at install time.
		return "Waiting for process to exit...";
	}

	public String getUninstallStatusMessage()
	{
		// Nothing at uninstall time.
		return "";
	}

	public boolean inTaskList (String wfpe_name) throws InstallException
	{
		Runtime Rt;
		String line;
		InputStream tasklist;
		BufferedReader taskreader;

		try
		{
			// Get runtime.
			Rt = Runtime.getRuntime();
			// Get task list by executing "tasklist"
			tasklist = Rt.exec ("tasklist").getInputStream ();
			// Read the output.
			taskreader = new BufferedReader(new InputStreamReader(tasklist));

			while ((line = taskreader.readLine()) != null)
			{
				if(line.indexOf(wfpe_name) >= 0)
					return true;
			}
			// We didn't see the task.
			return false;
		}
		catch (Exception e)
		{
			throw new NonfatalInstallException ("Reading tasklist failed.");
		}
	}

	public void install(InstallerProxy ip) throws InstallException
	{
		int iterations;
		String wfpe_name = ip.substitute ("$WFPE_NAME$");
		String wfpe_timeout = ip.substitute ("$WFPE_TIMEOUT$");

		System.out.println("Process: " + wfpe_name);

		try
		{
			iterations = Integer.parseInt (wfpe_timeout);
		}
		catch (Exception e)
		{
			// NumbeFormatException!
			throw new NonfatalInstallException ("Invalid timeout specified.");
		}

		System.out.println("Iterations: " + Integer.toString(iterations));


		while (inTaskList (wfpe_name))
		{
			try
			{
				Thread.sleep (sleep_1s);
			}
			catch (Exception e)
			{
				// If our sleep is interrupted just continue...
				// Print the exception just incase someone is logging this.
				System.out.println ("Exception: " + e );
			}
			iterations--;
			if (iterations == 0)
			{
				throw new NonfatalInstallException ("Timeout exceeded.");
			}
		}
		System.out.println("Exited with iterations = " + Integer.toString(iterations));
	}

	public void uninstall( UninstallerProxy up ) throws InstallException
	{
		// Do nothing.
	}
}
